/****************************************************************************

  Header file for template service
  based on the Gen 2 Events and Services Framework

 ****************************************************************************/

#ifndef ServFollower_H
#define ServFollower_H

#include <xc.h>
#include "ES_Types.h"
#include "PIC32_SPI_HAL.h"
#include "PIC32PortHAL.h"

#define SPI_MODULE SPI_SPI1

// Public Function Prototypes

bool InitFollowerService(uint8_t Priority);
bool PostFollowerService(ES_Event_t ThisEvent);
ES_Event_t RunFollowerService(ES_Event_t ThisEvent);
void PrintCmdDesc(uint8_t command);

typedef enum {
    InitFollower = 0,
    ExpectCommand,
    KeepCommand        
} FollowerState_t;

static void InitSPI(void) {
    SPISetup_BasicConfig(SPI_MODULE);
    SPISetup_SetFollower(SPI_MODULE);

    SPISetup_MapSSInput(SPI_MODULE, SPI_RPA0); // 2, RA0
    SPISetup_MapSDInput(SPI_MODULE, SPI_RPB8); // 17, RB8
    SPISetup_MapSDOutput(SPI_MODULE, SPI_RPA2); // 9, RA2

    SPI1BUF; // Clear buffer

    SPI1STATbits.SPIROV = 0;
    SPISetEnhancedBuffer(SPI_MODULE, true);
    SPISetup_SetXferWidth(SPI_MODULE, SPI_8BIT);
    SPISetup_SetActiveEdge(SPI_MODULE, SPI_FIRST_EDGE); // CKE = 0
    SPISetup_SetClockIdleState(SPI_MODULE, SPI_CLK_LO); // CKP = 1
    SPI1CONbits.SRXISEL = 0b01;

    IPC7bits.SPI1IP = 5;
    IFS1CLR = _IFS1_SPI1RXIF_MASK;
    IEC1SET = _IEC1_SPI1RXIE_MASK;

    SPISetup_EnableSPI(SPI_MODULE);
}

#endif /* ServFollower_H */

