
#include <xc.h>
#include "PIC32PortHAL.h"
#include "bitdefs.h"
#include <stdbool.h>

//values for all pins set for all ports
const uint32_t A_pins = _Pin_0|_Pin_1|_Pin_2|_Pin_3|_Pin_4;
const uint32_t B_pins = _Pin_0|_Pin_1|_Pin_2|_Pin_3|_Pin_4|_Pin_5|_Pin_6|_Pin_7|_Pin_8|_Pin_9|_Pin_10|_Pin_11|_Pin_12|_Pin_13|_Pin_14|_Pin_15;

//values for all pins set for ports that support analog input
const uint32_t AnalogIn_A = _Pin_0|_Pin_1;
const uint32_t AnalogIn_B = _Pin_0|_Pin_1|_Pin_2|_Pin_3|_Pin_12|_Pin_13|_Pin_14|_Pin_15;

bool PortSetup_ConfigureDigitalOutputs(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & B_pins) == Pins)){    //check if pins are valid for port B
        ANSELB &= ~Pins;    //clear ANSEL register for digital I/O
        TRISB &= ~Pins;     //clear TRIS register to enable output
        return 1;
    }
    if (Port==_Port_A && ((Pins & A_pins) == Pins)){    //check if pins are valid for port A
        ANSELA &= ~Pins;    
        TRISA &= ~Pins;
        return 1;
    }
    return 0;
}

bool PortSetup_ConfigureDigitalInputs(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & B_pins) == Pins)){    //check if pins are valid for port B
        ANSELBCLR = Pins;   //clear ANSEL register for digital I/O
        TRISBSET = Pins;    //set TRIS register to enable input
        return 1;
    }
    if (Port==_Port_A && ((Pins & A_pins) == Pins)){    //check if pins are valid for port A
        ANSELACLR = Pins;
        TRISASET = Pins;
        return 1;
    }
    return 0;
}

bool PortSetup_ConfigureAnalogInputs(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & AnalogIn_B) == Pins)){    //check if pins are valid analog inputs for port B
        TRISB |= Pins;  //set TRIS register to enable input
        return 1;
    }
    if (Port==_Port_A && ((Pins & AnalogIn_A) == Pins)){    //check if pins are valid analog inputs for port A
        TRISA |= Pins;
        return 1;
    }
    return 0;
}

bool PortSetup_ConfigurePullUps(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & B_pins) == Pins)){    //check if pins are valid for port B
        CNPUB |= Pins;  //set pull up register to enable internal pull up
        return 1;
    }
    if (Port==_Port_A && ((Pins & A_pins) == Pins)){    //check if pins are valid for port A
        CNPUA |= Pins;
        return 1;
    }
    return 0;
}

bool PortSetup_ConfigurePullDowns(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & B_pins) == Pins)){    //check if pins are valid for port B
        CNPDB |= Pins;  //set pull down register to enable internal pull down
        return 1;
    }
    if (Port==_Port_A && ((Pins & A_pins) == Pins)){    //check if pins are valid for port A
        CNPDA |= Pins;
        return 1;
    }
    return 0;
}

bool PortSetup_ConfigureOpenDrain(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & B_pins) == Pins)){    //check if pins are valid for port B
        ODCB |= Pins;   //set open drain register to configure open drain output
        return 1;
    }
    if (Port==_Port_A && ((Pins & A_pins) == Pins)){    //check if pins are valid for port A
        ODCA |= Pins;
        return 1;
    }
    return 0;
}

bool PortSetup_ConfigureChangeNotification(PortSetup_Port_t Port, PortSetup_Pin_t Pins){
    if (Port==_Port_B && ((Pins & B_pins) == Pins)){    //check if pins are valid for port B
        CNCONB |= BIT15HI;  //set BIT 15 in CNCON register to enable change notice control
        CNENB |= Pins;      //set CNEN register to enable CN interrupt
        return 1;
    }
    if (Port==_Port_A && ((Pins & A_pins) == Pins)){    //check if pins are valid for port A
        CNCONA |= BIT15HI;
        CNENA |= Pins;
        return 1;
    }
    return 0;
}