/****************************************************************************
Module
  EventCheckers.c

Revision
  1.0.1

Description
  This is the sample for writing event checkers along with the event
  checkers used in the basic framework test harness.

Notes
  Note the use of static variables in sample event checker to detect
  ONLY transitions.
 
History
When           Who     What/Why
-------------- ---     --------
08/06/13 13:36 jec     initial version
 ****************************************************************************/

// this will pull in the symbolic definitions for events, which we will want
// to post in response to detecting events
#include "ES_Configure.h"
//this gives us PostAll
#include "ES_Framework.h"
// this will get us the structure definition for events, which we will need
// in order to post events in response to detecting events
#include "ES_Events.h"
// if you want to use distribution lists then you need those function
// definitions too.
#include "ES_PostList.h"
// This include will pull in all of the headers from the service modules
// providing the prototypes for all of the post functions
#include "ES_ServiceHeaders.h"
// this test harness for the framework references the serial routines that
// are defined in ES_Port.c
#include "ES_Port.h"
// include our own prototypes to insure consistency between header &
// actual functions definition
#include "EventCheckers.h"

// include our own prototypes to insure consistency between header &
// actual functions definition
#include "SwitchChecker.h"

//for debugging, print onto the screen
#include "terminal.h"
#include "dbprintf.h"

void SwitchInit(void) {

    //set switch as digital input RB11/RB12

    //clear ANSELx to give pin digital capacity
    ANSELB &= ~(1 << 11);
    //configure the specified pin as Digital Input using TRISx
    TRISB |= (1 << 11);
    //set as pull down to prevent pin from floating
    CNPDB |= (1 << 11);
    
    ANSELB &= ~(1 << 12);
    //configure the specified pin as Digital Input using TRISx
    TRISB |= (1 << 12);
    //set as pull down to prevent pin from floating
    CNPDB |= (1 << 12);

    return;
}

bool FrontSwitchChecker(void) {

    //local RetrunVal = False
    bool ReturnVal = false;

    //create static variable to check switch reading
    //input port is RB11
    static uint32_t LastInputState = 0;
    uint32_t CurrentInputState = 0;
    CurrentInputState = PORTBbits.RB11;

    //If the state of the input line has changed
    if (!(CurrentInputState == LastInputState)) {
        //If the current state of the input line is HIGH, switch has pull down
        if (CurrentInputState == 1) {

            DB_printf("\rFrontLimitSwitch changed to %d\r\n", CurrentInputState);

            //PostEvent FrontCollision with parameter of the state
            ES_Event_t FrontCollision = {HW_FRONTWALLCOLLIDE, CurrentInputState};
            PostMotor(FrontCollision);

        } else { //Else (current input state is low), no longer contacting the wall 

            DB_printf("\rFrontLimitSwitch changed to %d\r\n", CurrentInputState);

        }
        ReturnVal = true;
    }

    //Set LastInputState to CurrentInputState 
    LastInputState = CurrentInputState;

    return ReturnVal;
}

bool RearSwitchChecker(void) {

    //local RetrunVal = False
    bool ReturnVal = false;

    //create static variable to check switch reading
    //input port is RB12
    static uint32_t LastInputState = 0;
    uint32_t CurrentInputState = 0;
    CurrentInputState = PORTBbits.RB12;

    //If the state of the input line has changed
    if (!(CurrentInputState == LastInputState)) {
        //If the current state of the input line is HIGH, switch has pull down
        if (CurrentInputState == 1) {

            DB_printf("\rRearLimitSwitch changed to %d\r\n", CurrentInputState);

            //PostEvent RearCollision with parameter of the state
            ES_Event_t RearCollision = {HW_REARWALLCOLLIDE, CurrentInputState};
            PostMotor(RearCollision);

        } else { //Else (current input state is low), no longer contacting the wall 

            DB_printf("\rRearLimitSwitch changed to %d\r\n", CurrentInputState);

        }
        ReturnVal = true;
    }

    //Set LastInputState to CurrentInputState 
    LastInputState = CurrentInputState;

    return ReturnVal;
}
