/****************************************************************************
 Module
     main.c
 Description
     starter main() function for Events and Services Framework applications
 Notes

*****************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>

#include "ES_Configure.h"
#include "ES_Framework.h"
#include "ES_Port.h"

// stripped down printf()
#include "dbprintf.h"

/** INCLUDES ******************************************************************/

#include <xc.h>             /* contains Vector Name/Number Macros */
#include <sys/attribs.h>    /* contains __ISR() Macros */

// Exception ExcCode Macros (see Table 50-12 in FRM Section-50 CPU - DS61192)

#define EXCCODE_ADDRESS_ERROR_STORE 5
#define EXCCODE_TRAP_EXCEPTION 13

/** VARIABLES *****************************************************************/

int *error_1;       // pointer variable used to trigger illegal address exception
int error_2;        // variable used to trigger trap (divide-by-zero) exception

/* Exception Handler Variables */
/* Static in case exception condition would stop auto variable from being created */
static unsigned int _excep_code;
static unsigned int _excep_addr;


#define clrScrn() printf("\x1b[2J")
#define goHome() printf("\x1b[1,1H")
#define clrLine() printf("\x1b[K")

void main(void)
{
  ES_Return_t ErrorType;

  _HW_PIC32Init();
  clrScrn();

  // When doing testing, it is useful to announce just which program
  // is running.
  puts("\rStarting Test Harness for \r");
  printf( "FOLLOWER\r\n");
  printf( "%s %s\n", __TIME__, __DATE__);
  printf( "\n\r\n");
  printf( "Press any key to post key-stroke events to Service 0\n\r");
  printf( "Press 'd' to test event deferral \n\r");
  printf( "Press 'r' to test event recall \n\r");

  // Your hardware initialization function calls go here

  // now initialize the Events and Services Framework and start it running
  ErrorType = ES_Initialize(ES_Timer_RATE_1mS);
  if (ErrorType == Success)
  {
    ErrorType = ES_Run();
  }
  //if we got to here, there was an error
  switch (ErrorType)
  {
    case FailedPost:
    {
      printf("Failed on attempt to Post\n");
    }
    break;
    case FailedPointer:
    {
      printf("Failed on NULL pointer\n");
    }
    break;
    case FailedInit:
    {
      printf("Failed Initialization\n");
    }
    break;
    default:
    {
      printf("Other Failure\n");
    }
    break;
  }
  for ( ; ;)
  {
    ;
  }
}

void __attribute__((nomips16)) _general_exception_handler(void)
{
    // Mask off the ExcCode Field from the Cause Register
    // Refer to FRM Section-2 CPU - DS61113
    _excep_code=(_CP0_GET_CAUSE() & 0x0000007C) >> 2;
    _excep_addr=_CP0_GET_EPC();

    // Disable Interrupts
    _CP0_SET_STATUS(_CP0_GET_STATUS()&0xFFFFFFE); 

    // Examine _excep_code to identify the type of exception
    // Examine _excep_addr to find the address that caused the exception
    // Perform an indication of the exception
    switch(_excep_code)
    {
        case EXCCODE_ADDRESS_ERROR_STORE:
//            LED_LD6_SET();
            break;
            
        case EXCCODE_TRAP_EXCEPTION:
//            LED_LD5_SET();
            break;
            
        default:
            // Set both LEDs on any other exception
//            LED_LD6_SET();
//            LED_LD5_SET();
            break;
    }
    
    while(1);       // Set a breakpoint here
                    // Examine _excep_code to identify the type of exception
                    // Examine _excep_addr to find the address that caused the exception
                    // Example LEDs for indication

                    // In a real application, you would perform a useful indication
                    // of the exception, as well as a recovery mechanism
}